Imports System.Text
Imports System.Net
Imports System.Net.Sockets
Imports System.Threading

Public Class IRC
    Private sEmail As String
    Private userHash As New Hashtable()
    Private m_userhash As New Hashtable()
    Private WithEvents IRCClient As New IRCSocket()

#Region " Properties "

    Private bChannelModerated As Boolean = False
    Private bConnected As Boolean = False
    Private sServer As String
    Private iPort As Integer
    Private sRoom As String
    Private sNick As String
    Private sAltNick As String

    Public Property Server() As String
        Get
            Server = sServer
        End Get
        Set(ByVal Value As String)
            sServer = Value
        End Set
    End Property
    Public Property Port() As Integer
        Get
            Port = iPort
        End Get
        Set(ByVal Value As Integer)
            iPort = Value
        End Set
    End Property
    Public Property Room() As String
        Get
            Room = sRoom
        End Get
        Set(ByVal Value As String)
            sRoom = Value
        End Set
    End Property
    Public ReadOnly Property IsConnected() As Boolean
        Get
            IsConnected = bConnected
        End Get
    End Property

    Public Property NickName() As String
        Get
            NickName = sNick
        End Get
        Set(ByVal Value As String)
            sNick = Value
        End Set
    End Property

    Public Property AltNickName() As String
        Get
            AltNickName = sAltNick
        End Get
        Set(ByVal Value As String)
            sAltNick = Value
        End Set
    End Property

    Public Property Email() As String
        Get
            Email = sEmail
        End Get
        Set(ByVal Value As String)
            sEmail = Value
        End Set
    End Property

#End Region

#Region " Methods "

    Public Sub ConnectToServer()
        Try
            If sServer = "" Or iPort = 0 Or sNick = "" Or sAltNick = "" Or sEmail = "" Or bConnected = True Then Exit Sub
            IRCClient.RemoteHost = sServer
            IRCClient.RemotePort = iPort
            IRCClient.Connect()
        Catch e As Exception
            RaiseEvent IRC_Error(e)
        End Try
    End Sub

    Public Sub DisconnectFromServer()
        If bConnected = True Then
            Try
                IRCClient.SendString("QUIT" & vbCrLf)
                IRCClient.Disconnect()
                bConnected = False
            Catch e As Exception

            End Try
        Else
            RaiseEvent IRC_Disconnected()
        End If
    End Sub

    Public Function CheckIfOp(ByVal user As String) As Boolean
        'some networks use different identifiers, this simply checks them all
        If m_userhash.ContainsKey("&" & user) Or m_userhash.ContainsKey("@" & user) Or m_userhash.ContainsKey("~" & user) Or m_userhash.ContainsKey("*" & user) Or m_userhash.ContainsKey("^" & user) Then
            Return True
        Else
            Return False
        End If
    End Function

    Public Function CheckIfHalfOp(ByVal user As String) As Boolean
        Return m_userhash.ContainsKey("%" & user)
    End Function

    Public Function CheckIfVoice(ByVal user As String) As Boolean
        Return m_userhash.ContainsKey("+" & user)
    End Function
#End Region

#Region " Events "
    Public Event IRC_Connected()
    Public Event IRC_Disconnected()
    Public Event IRC_Dropped()
    Public Event IRC_Raw_Recieved(ByVal RawData As String)
    Public Event IRC_MOTD(ByVal MOTD As String)
    Public Event IRC_PM(ByVal user As String, ByVal message As String)
    Public Event IRC_Page(ByVal user As String, ByVal message As String)
    Public Event IRC_Chat(ByVal user As String, ByVal message As String)
    Public Event IRC_Topic(ByVal NewTopic As String)
    Public Event IRC_Join(ByVal user As String)
    Public Event IRC_Quit(ByVal user As String, ByVal reason As String)
    Public Event IRC_Part(ByVal user As String, ByVal reason As String)
    Public Event IRC_Kick(ByVal user As String, ByVal KickedBy As String, ByVal reason As String)
    Public Event IRC_Mode(ByVal user As String, ByVal mode As String, ByVal ChangedBy As String)
    Public Event IRC_Notice(ByVal user As String, ByVal message As String)
    Public Event IRC_NickChange(ByVal user As String, ByVal NewNick As String)
    Public Event IRC_Error(ByVal e As Exception)
    Public Event IRC_GotUserList()
    Public Event IRC_IRC_Error(ByVal message As String)
    Public Event IRC_Channel_Invite(ByVal invitedby As String, ByVal channel As String)
    Public Event IRC_WallChop(ByVal user As String, ByVal message As String, ByVal channel As String)
#End Region

#Region " Internal Functions & Events "

    Private Sub IRCClient_Connected() Handles IRCClient.Connected
        bConnected = True
        IRCClient.SendString("NICK " & sNick & vbCrLf)
        IRCClient.SendString("USER " & sNick & " localhost 0.0.0.0 :BR .NET" & vbCrLf)
    End Sub

    Private Sub IRCClient_Disconnected() Handles IRCClient.Disconnected
        bConnected = False
        RaiseEvent IRC_Disconnected()
    End Sub

    Private Sub IRCClient_DisconnectedByRemote() Handles IRCClient.DisconnectedByRemote
        bConnected = False
        RaiseEvent IRC_Dropped()
    End Sub

    Private Sub IRCClient_DataArrival(ByVal strString As String) Handles IRCClient.DataArrival
        Dim inData As String = strString
        Dim lData() As String
        Dim C() As String
        Dim tData As Integer
        Dim CL As Integer
        Dim CLi As String
        Dim a As Integer

        lData = Split(inData, Chr(10))
        tData = UBound(lData) - LBound(lData)
        For a = 0 To tData
            If Mid(lData(a), 1, 1) = ":" Then
                lData(a) = Mid(lData(a), 2)
            End If
        Next
        For CL = 0 To tData
            C = Split(lData(CL), " ")
            ReDim Preserve C(2000)
            CLi = lData(CL)
            If C(0) = "PING" Then
                IRCClient.SendString("PONG :" & Mid(C(1), 2, Len(C(1)) - 2) & vbCrLf)
            End If
            If C(0) = "ERROR" Then 'ERROR :Closing Link:
                IRCClient.Disconnect()
                RaiseEvent IRC_Dropped()
            End If
            If C(1) <> Nothing Then
                Select Case C(1).ToUpper
                    Case "376", "422" 'connected
                        RaiseEvent IRC_Connected()
                        IRCClient.SendString("MODE " & Me.Room & vbCrLf)
                    Case "372" 'motd
                        RaiseEvent IRC_MOTD(Combined(C, 3).Substring(2))
                    Case "353" 'names
                        Dim tmp() As String = Split(Combined(C, 5), " ", C.IndexOf(C, Chr(10)))
                        Dim i As Integer
                        For i = 0 To UBound(tmp)
                            If i = 0 Then
                                Me.userHash.Add(tmp(i).Substring(1), Me.Room)
                            Else
                                Me.userHash.Add(tmp(i), Me.Room)
                            End If
                        Next
                    Case "366" 'end of names
                        Me.m_userhash = userHash.Clone()
                        userHash.Clear()
                        RaiseEvent IRC_GotUserList()
                    Case "332" 'channel topic
                        RaiseEvent IRC_Topic(Combined(C, 4).Substring(1))
                    Case "319" 'Added to allow determining if a person is an op on the channel
                    Case "324" 'modes return
                        If InStr(C(4), "m") Then
                            If Me.CheckIfOp(Me.NickName) = False Then
                                Me.bChannelModerated = True
                            Else
                                Me.bChannelModerated = False
                            End If
                        Else
                            Me.bChannelModerated = False
                        End If
                    Case "PRIVMSG"
                        Dim message As String
                        message = Combined(C, 3).Substring(1)
                        If message.Substring(0, 1) = Chr(1) Then
                            Dim ctcp As String = C(3).Substring(2)
                            If Not C(4) Is Nothing Then
                                message = Combined(C, 4)
                                message = message.Substring(0, message.Length - 1)
                            Else
                                message = ""
                                ctcp = ctcp.Substring(0, ctcp.Length - 2)
                            End If
                            Me.Ctcps(StripUserFromInfo(C(0)), sRoom, ctcp, message)
                        Else
                            'check if room message, or private message
                            If C(2).ToLower = Me.sNick.ToLower Or C(2).ToLower = Me.sAltNick.ToLower Then
                                RaiseEvent IRC_PM(StripUserFromInfo(C(0)), Combined(C, 3).Substring(1))
                            Else
                                RaiseEvent IRC_Chat(StripUserFromInfo(C(0)), Combined(C, 3).Substring(1))
                            End If
                        End If
                    Case "KICK"
                        RaiseEvent IRC_Kick(C(3), StripUserFromInfo(C(0)), Combined(C, 4).Substring(1))
                        IRCClient.SendString("NAMES " & Me.Room & vbCrLf)
                    Case "QUIT"
                        RaiseEvent IRC_Quit(StripUserFromInfo(C(0)), Combined(C, 1))
                        IRCClient.SendString("NAMES " & Me.Room & vbCrLf)
                    Case "PART"
                        RaiseEvent IRC_Part(StripUserFromInfo(C(0)), Combined(C, 3))
                        IRCClient.SendString("NAMES " & Me.Room & vbCrLf)
                    Case "NICK"
                        RaiseEvent IRC_NickChange(StripUserFromInfo(C(0)), C(2).Substring(1).Trim)
                        IRCClient.SendString("NAMES " & Me.Room & vbCrLf)
                    Case "JOIN"
                        Dim tmp As String
                        tmp = StripUserFromInfo(C(0))
                        If tmp <> Me.sNick And tmp <> Me.sAltNick Then
                            RaiseEvent IRC_Join(StripUserFromInfo(C(0)))
                        End If
                        IRCClient.SendString("NAMES " & Me.Room & vbCrLf)
                    Case "MODE"
                        'something changed, update the names list and get channel modes
                        IRCClient.SendString("NAMES " & Me.Room & vbCrLf)
                        IRCClient.SendString("MODE " & Me.Room & vbCrLf)
                    Case "NOTICE"
                        'check for wallchops (@channel)
                        If C(2) = "@" & Me.Room Then
                            RaiseEvent IRC_WallChop(StripUserFromInfo(C(0)), Combined(C, 3).Substring(1), Me.Room.Substring(1))
                        Else
                            RaiseEvent IRC_Notice(StripUserFromInfo(C(0)), Combined(C, 3).Substring(1))
                        End If
                    Case "TOPIC"
                        RaiseEvent IRC_Topic(Me.Combined(C, 3).Substring(1).Trim)
                    Case "INVITE"
                        RaiseEvent IRC_Channel_Invite(StripUserFromInfo(C(0)), Me.Combined(C, 3).Substring(1).Trim)
                    Case "404" 'errors
                        RaiseEvent IRC_IRC_Error(Combined(C, 3).Substring(1).Trim)
                    Case "421" 'unknown command
                    Case Else
                        RaiseEvent IRC_Raw_Recieved(CLi)
                End Select
                RaiseEvent IRC_Raw_Recieved(CLi)
            End If
        Next
    End Sub


    Private Function StripUserFromInfo(ByVal userinfo As String) As String
        If InStr(userinfo, "!") Then
            StripUserFromInfo = Mid(userinfo, 1, InStr(userinfo, "!") - 1)
        Else
            StripUserFromInfo = userinfo
        End If
    End Function

    Private Function Combined(ByVal Sourcearray() As String, ByVal StartPosition As Integer) As String
        Try
            If StartPosition <= Sourcearray.Length - 1 Then
                Dim result As String
                Return result.Join(" ", Sourcearray, StartPosition, Sourcearray.Length - StartPosition).Trim
            Else
                Return ""
            End If
        Catch
        End Try
    End Function

#End Region

#Region " Ctcp Stuff "
    'enter these values here...
    Private Ctcp_Version As String = ""
    Private CTCP_Finger As String = ""
    Private CTCP_CleintInfo As String = "ACTION CLIENTINFO FINGER PING TIME USERINFO VERSION"
    Private CTCP_UserInfo As String = ""

    Public Property Version() As String
        Get
            Return Me.Ctcp_Version
        End Get
        Set(ByVal Value As String)
            Me.Ctcp_Version = Value
        End Set
    End Property
    Public Property Finger() As String
        Get
            Return Me.CTCP_Finger
        End Get
        Set(ByVal Value As String)
            Me.CTCP_Finger = Value
        End Set
    End Property
    Public Property ClientInfo() As String
        Get
            Return Me.CTCP_UserInfo
        End Get
        Set(ByVal Value As String)
            Me.CTCP_UserInfo = Value
        End Set
    End Property
    Public Property UserInfo() As String
        Get
            Return Me.CTCP_UserInfo
        End Get
        Set(ByVal Value As String)
            Me.CTCP_UserInfo = Value
        End Set
    End Property

    'hopfully I can build most of the functionially of ctcp should be built into
    'here I hope.
    Sub Ctcps(ByVal Userinfo As String, ByVal channel As String, ByVal Ctcp As String, ByVal message As String)
        Try
            Select Case LCase(Ctcp)
                Case "version"
                    Notice(Chr(1) + "Version " + Ctcp_Version + Chr(1), Userinfo)
                Case "ping"
                    Notice(Chr(1) + "Ping " + message + Chr(1), Userinfo)
                Case "time"
                    Notice(Chr(1) + "Time " + CStr(Now) + Chr(1), Userinfo)
                Case "finger"
                    Notice(Chr(1) + "Finger " & CTCP_Finger + Chr(1), Userinfo)
                Case "userinfo"
                    Notice(Chr(1) + "Userinfo " + CTCP_UserInfo + Chr(1), Userinfo)
                Case "clientinfo"
                    Notice(Chr(1) + "Clientinfo " + CTCP_CleintInfo + Chr(1), Userinfo)
                Case "page"
                    RaiseEvent IRC_Page(Userinfo, message)
                Case Else
                    'servobj.Notice (Chr(1) + "ERRMSG" + Message + Chr(1)), nickname
            End Select
        Catch
        End Try
    End Sub

#End Region

#Region " Message Types "
    'this is just a helper class to quickly allow access to common
    'message responses. The irc uses.

    Public Sub Message(ByVal message As String, ByVal channel As String)
        If Me.bChannelModerated = True And CheckIfOp(Me.NickName) = False Then
            'nothing, channel is moderated, as we aren't op
        Else
            IRCClient.SendString("PRIVMSG " + channel + " :" + message & vbCrLf)
        End If
    End Sub

    Public Sub SendPM(ByVal message As String, ByVal user As String)
        IRCClient.SendString("PRIVMSG " & user & " :" & message & vbCrLf)
    End Sub

    Public Sub Notice(ByVal message As String, ByVal channel As String)
        IRCClient.SendString("Notice " + channel + " :" + message & vbCrLf)
    End Sub

    Public Sub Action(ByVal MessageS As String, ByVal channel As String)
        If Me.bChannelModerated = True And CheckIfOp(Me.NickName) = False Then
            'nothing, channel is moderated, as we aren't op
        Else
            MessageS = (Chr(1) & "ACTION " & MessageS & Chr(1))
            IRCClient.SendString("PRIVMSG " & channel & " :" & MessageS & vbCrLf)
        End If
    End Sub

    Public Sub Kick(ByVal NickName As String, ByVal channel As String)
        IRCClient.SendString("Kick " + channel + " :" + NickName & vbCrLf)
    End Sub

    Public Sub Join(ByVal channels As String, ByVal chankey As String)
        sRoom = channels

        If chankey.Length = 0 Then
            IRCClient.SendString("JOIN " + channels & vbCrLf)
        Else
            IRCClient.SendString("JOIN " + channels + " " + chankey & vbCrLf)
        End If
    End Sub

    Public Sub Part(ByVal channels As String, ByVal Reason As String)
        IRCClient.SendString("part " + channels + " " + Reason & vbCrLf)
    End Sub

    Public Sub Quit(ByVal Reason As String)
        IRCClient.SendString("QUIT :" + Reason & vbCrLf)
    End Sub

    Public Sub Away(ByVal Reason As String)
        IRCClient.SendString("Away :" + Reason & vbCrLf)
    End Sub

    Public Sub Modes(ByVal changes As String)
        IRCClient.SendString("mode " & changes & vbCrLf)
    End Sub

    Public Sub Nick(ByVal NewNick As String)
        IRCClient.SendString("Nick " + NewNick & vbCrLf)
    End Sub

    Public Sub Whois(ByVal NickName As String)
        IRCClient.SendString("Whois " & NickName & vbCrLf)
    End Sub

    Public Sub Who(ByVal NickName As String)
        IRCClient.SendString("Who " & NickName & vbCrLf)
    End Sub

    Public Sub NSIdentify(ByVal password As String, ByVal ServType As String)
        Select Case (ServType)
            Case "NickServ"
                IRCClient.SendString("NICKSERV GHOST " & sNick & " " & password & vbCrLf)
                IRCClient.SendString("NICK " & sNick & vbCrLf)
                IRCClient.SendString("NICKSERV IDENTIFY " & password & vbCrLf)
            Case "AuthServ"
                IRCClient.SendString("AUTHSERV GHOST " & sNick & " " & password & vbCrLf)
                IRCClient.SendString("NICK " & sNick & vbCrLf)
                IRCClient.SendString("AUTHSERV AUTH " & password & vbCrLf)
            Case "X"
                IRCClient.SendString("PRIVMSG X login " & sNick & " " & password & vbCrLf)
            Case "Q"
                IRCClient.SendString("PRIVMSG Q :AUTH " & sNick & " " & password & vbCrLf)
        End Select
    End Sub

    Public Sub Invite(ByVal nick As String, ByVal channel As String)
        IRCClient.SendString("INVITE " & nick & " " & channel & vbCrLf)
    End Sub

    Public Sub WallChops(ByVal message As String, ByVal channel As String)
        IRCClient.SendString("NOTICE @" & channel & " :" & message & vbCrLf)
    End Sub
#End Region


End Class

Public Class IRCSocket
    'optimized version of jSocket, this is very clean and fast

#Region " Private Stuff "
    Private Listener As TcpClient
    Private lThread As Thread
    Private bConnected As Boolean = False
    Private ns As NetworkStream
#End Region

#Region " Public Variables "
    Public RemoteHost As String = ""
    Public RemotePort As Integer = 0
#End Region

#Region " Events "
    Public Event DisconnectedByRemote()
    Public Event DataArrival(ByVal sData As String)
    Public Event Connected()
    Public Event Disconnected()
#End Region

#Region " Methods "
    Public Sub New()
        'class initialiation
    End Sub

    Public Sub Connect()
        lThread = New Thread(AddressOf Listen)
        bConnected = True
        lThread.Start()
    End Sub

    Public Sub SendString(ByVal strString As String)
        If ns.CanWrite Then
            ns.Write(Encoding.ASCII.GetBytes(strString), 0, Encoding.ASCII.GetByteCount(strString))
        Else
            RaiseEvent DisconnectedByRemote()
            bConnected = False
        End If
    End Sub

    Public Sub Disconnect()
        Listener.Close()
        bConnected = False
    End Sub

    Protected Overrides Sub Finalize()
        Listener = Nothing
        lThread = Nothing
        MyBase.Finalize()
    End Sub

#End Region

#Region " Socket Coding "
    Private Sub Listen()
        Listener = New TcpClient(RemoteHost, RemotePort)
        ns = Listener.GetStream
        RaiseEvent Connected()
        Try
ReadStart:
            Do While bConnected = True
                Dim buffer(4095) As Byte, retCount = 0
                retCount = ns.Read(buffer, 0, 4096)
                ReDim Preserve buffer(retCount)
                RaiseEvent DataArrival(Encoding.ASCII.GetString(buffer))
            Loop
            RaiseEvent Disconnected()
        Catch
            If ns.CanRead And bConnected Then
                GoTo ReadStart
            Else
                RaiseEvent DisconnectedByRemote()
            End If
        End Try
    End Sub
#End Region

End Class
