Imports System.Threading
Imports System.Net.Sockets
Imports System.Text

'Daves Web Server Classes - Important Notes, Please Read

'This file contains 3 classes used to control an application
'based web server. It has been heavily modified to work with
'Blaze Regulator. I am going to detail the URLs you need to
'use to retrieve information from the server.

'An embedded file, any file of any type you have embedded
'within your project, can be retrieved from the file using the
'url "/embed/<filename.ext>". This will allow you to include
'simple HTML pages and Image files within your project, and
'return them from the web server.

'Another great feature i've prepared for you is the ability
'to call any sub/function in your project that is accessible 
'from the WebClient class. To do this, request a URL like
'this: "/function/<functionname>?arg1=<arg1>&arg2=<arg2>" 
'When using "Function", it will send the returned data as
'HTML to the user. So you can make Functions that return
'generated web pages. (You MUST call a function, not a Sub)

'Currently, this feature is limited to STRINGs only. If you
'attempt to send a variable to a function that requires an
'integer, you will get an error. You must make sure your
'function is set to get its variables As String, and then
'convert them in your procedure afterwards. (This is a limit
'from getting the values from a URL.)

'Note: This feature has been made to ONLY execute code from 
'class "clsWebExposedProcedures", this prevents people being 
'able to execute code from the App itself, since the CallByName 
'command used by this feature is very powerful.

'The server will also return generated HTML directory listings,
'that can be used to return log files, etc. The only URL i have
'set up for you, for security reasons, is "/hd/logs", which you
'may configure to open any folder you choose.

'Please do *not* alter any code that has been enclosed within any
'#Region blocks. These control the web server. There are
'functions not inside #Region blocks that you may alter to
'control how the Web Server responds.

'---------------------------------------------------------------
'To use this Web Server class, simpily create a new instance
'of clsWebServer, using the required port number as a parameter.
'---------------------------------------------------------------

Public Class clsWebServer

#Region " Variable Declarations "


    Const iPortNumber As Integer = 2311

    Private mobjServer As Object
    Private mobjWebServerThread As Thread
    Private mobjWebListener As TcpListener

    Public mcolWebClients As New Hashtable()

    Public ValidUsername As String = "Administrator"
    Public ValidPassword As String = "123456"

#End Region

#Region " Winsock Listening Code "

    Public Sub New(ByVal Server As Object)
        mobjServer = Server
    End Sub

    Public Sub Listen()
        Try
            mobjWebServerThread = New Threading.Thread(AddressOf WebServer_Listen)
            mobjWebServerThread.Name = "WorkWatch WebServerThread"
            mobjWebServerThread.Start()
            Console.WriteLine("""" & mobjWebServerThread.Name & """ Started")
        Catch
            'bugger
        End Try
    End Sub

    Public Sub EndListen()
        Try
            mobjWebListener.Stop()
            mobjWebServerThread.Abort()
            Console.WriteLine(mobjWebServerThread.Name & " Aborted")
        Catch
            'bugger
        End Try
    End Sub

    Private Sub WebServer_Listen()
        Try
            mobjWebListener = New TcpListener(iPortNumber)
            mobjWebListener.Start()
            Do
                Console.WriteLine("New Client has Connected")
                'New Client has Connected
                Dim x As New clsWebClient(mobjWebListener.AcceptSocket(), mobjServer, Me)

                'We need to find a free Index number for this client
                Dim i As Integer = 0
                Dim iIndex As Integer
                Dim iObject As clsWebClient

                Do
                    i += 1
                    Try
                        'Attempt to read index value
                        iObject = mcolWebClients(CStr(i))
                        iIndex = iObject.Index
                    Catch
                        'When caught, "i" will be a free Index number
                        x.Index = i
                        Exit Do
                    End Try
                Loop

                'Console.WriteLine("Client Issued ID - " & x.Index & " {" & x.ID & "}")

                'Attach the thread to handle Terminating Sockets
                AddHandler x.SocketDisconnected, AddressOf WebClient_Disconnect

                'Attach new Client to Clients Collection
                mcolWebClients.Add(CStr(x.Index), x)
                'Console.WriteLine("Client Attached to Server")

            Loop Until False
        Catch e As Exception
            If Err.Number() <> 5 Then
                'If the Listening Thread wasnt Aborted...
                MsgBox(e.ToString)
            End If
        End Try

    End Sub

    Private Sub WebClient_Disconnect(ByVal Sender As Object)
        Try
            Sender.mobjSocket.Close()
            Sender.mobjSocket = Nothing
            mcolWebClients.Remove(Sender.Index)
        Catch
            'bugger
        End Try
    End Sub

#End Region

End Class

Public Class clsWebClient

    Private Function AuthenticateUser(ByVal Username As String, ByVal Password As String) As Boolean
        'This Function determines who can get access to the web server, you
        'need to validate the Username/Password against your own records.
        'Then return True if they are valid, or False if they are not.
        If Username = mobjWebServer.ValidUsername And Password = mobjWebServer.ValidPassword Then
            Return True
        Else
            Return False
        End If
    End Function

    Private Function SetHarddriveExposedPaths()
        'This function is used to determine where a client can reach
        'on the local harddrive from the web interface. Copy the
        'syntax used below to add more. Remember that all URLs are
        'automatically prefixed with "/hd".

        'Note: This function is called automatically, just add items 
        'to the list, that is all. (You may use variables for 
        'these paths too.)

        HarddrivePaths.Add("/logs|" & Application.StartupPath & "\logs\")

    End Function

#Region " Variable Declarations "

    Public StatusCode As String
    Public AppendHeader As String

    Private mgID As Guid = Guid.NewGuid
    Public Index As String = ""

    Private HarddrivePaths As System.Collections.Specialized.StringCollection = New System.Collections.Specialized.StringCollection()

    Dim ExposeFunctions As clsWebExposedProcedures

#End Region

#Region " Winsock Send/Receive Code "

    Private mobjServer As Object
    Private mobjWebServer As clsWebServer

    Private mobjClient As TcpClient
    Private mobjText As New StringBuilder()
    Private marData(1024) As Byte
    Public mobjSocket As Socket
    Private nsSocket As NetworkStream

    Private bDisconnect As Boolean = False

    Public Sub New(ByVal ClientSocket As Socket, ByVal Server As Object, ByVal WebServer As clsWebServer)
        'Attach tcpclient object to new Socket Class
        mobjSocket = ClientSocket

        'Attach Parent Server Object
        mobjServer = Server
        mobjWebServer = WebServer

        SetHarddriveExposedPaths()

        'mobjSocket.GetStream.BeginRead(marData, 0, 1024, AddressOf DoStreamReceive, Nothing)
        nsSocket = New NetworkStream(mobjSocket)
        nsSocket.BeginRead(marData, 0, 1024, AddressOf DoStreamReceive, Nothing)
    End Sub

    Private Sub BuildString(ByVal Bytes() As Byte, ByVal offset As Integer, ByVal count As Integer)
        Dim intIndex As Integer

        For intIndex = offset To offset + count - 1
            If intIndex = offset + count - 1 Then
                mobjText.Append(ChrW(Bytes(intIndex)))
                ProcessCommand(mobjText.ToString())
                mobjText = New StringBuilder()
            Else
                mobjText.Append(ChrW(Bytes(intIndex)))
            End If
        Next
    End Sub

    Private Sub DoStreamReceive(ByVal ar As IAsyncResult)
        Dim intCount As Integer

        Try
            SyncLock nsSocket
                intCount = nsSocket.EndRead(ar)
            End SyncLock

            If intCount < 1 Then
                'This is a "Terminate Connection" byte.
                'Disconnect Here
                bDisconnect = True
                RaiseEvent SocketDisconnected(Me)
                Exit Sub
            End If

            'Append our recieved bytes into a String
            BuildString(marData, 0, intCount)
        Catch e As Exception
            Console.WriteLine("Error")
            'RaiseEvent SocketDisconnected(Me)
        Finally
            'Recreate a new Reading Thread
            'mobjClient.GetStream.BeginRead(marData, 0, 1024, AddressOf DoStreamReceive, Nothing)
            If bDisconnect = False Then
                Try
                    If mobjSocket.Connected = True Then
                        SyncLock nsSocket
                            nsSocket = New NetworkStream(mobjSocket)
                            nsSocket.BeginRead(marData, 0, 1024, AddressOf DoStreamReceive, Nothing)
                        End SyncLock
                    End If
                Finally
                    'nothing
                End Try
            Else
                'Full Disconnect Here
                If Not mobjSocket Is Nothing Then
                    mobjSocket.Close()
                    mobjSocket = Nothing
                End If
            End If
        End Try
    End Sub

    Public Sub SendData(ByVal Data As String)
        Dim w As New IO.StreamWriter(nsSocket)

        w.Write(Data)
        w.Flush()
    End Sub

    Public ReadOnly Property RemoteHostIP() As String
        Get
            Dim IPE As System.Net.IPEndPoint
            IPE = mobjSocket.RemoteEndPoint
            Return IPE.Address.ToString
        End Get
    End Property

    Public Event SocketDisconnected(ByVal Sender As Object)

    Public Sub CloseMe()
        bDisconnect = True
        RaiseEvent SocketDisconnected(Me)
    End Sub

#End Region

#Region " Private Web Functions "

    Private Sub ProcessCommand(ByVal strCommand As String)
        Try
            'First split the command into various items
            Dim splCommand() As String = Split(strCommand, vbCrLf, -1, CompareMethod.Text)

            'Find out "METHOD" and "URI"
            Dim RequestLine() As String = Split(splCommand(0), " ", -1, CompareMethod.Text)
            Dim RequestMethod As String = RequestLine(0)
            Dim RequestURL As String = RequestLine(1)
            Dim RequestVersion As String = RequestLine(2)

            Select Case UCase(RequestMethod)
                Case "GET"
                    RequestURL = UrlDecode(RequestURL)
                    Request_Get(RequestURL, splCommand)
                Case Else
                    'Return Code 501 (Not Implemented)
            End Select

        Catch e As Exception
            Console.WriteLine(e.ToString)
        End Try
    End Sub

    Private Function UrlDecode(ByVal sEncoded As String) As String
        Dim pos As Long             ' position of InStr target

        If sEncoded = "" Then Exit Function
        ' convert "+" to space
        pos = 0
        Do
            pos = InStr(pos + 1, sEncoded, "+", CompareMethod.Text)
            If pos = 0 Then Exit Do
            Mid$(sEncoded, pos, 1) = " "
        Loop

        ' convert "%xx" to character
        pos = 0
        Do
            pos = InStr(pos + 1, sEncoded, "%", CompareMethod.Text)
            If pos = 0 Then Exit Do

            Mid$(sEncoded, pos, 1) = Chr("&H" & (Mid$(sEncoded, pos + 1, 2)))
            sEncoded = Left$(sEncoded, pos) & Mid$(sEncoded, pos + 3)
        Loop

        UrlDecode = sEncoded
    End Function

    Private Function Request_Get(ByVal strURL As String, ByVal strHeader() As String)
        'Check Authentication
        Dim AuthEncrypted As String
        Dim strOK As Boolean = False
        Dim bIsEmbedded As Boolean = False

        Dim HeaderInfo As String
        Dim Z As Integer
        For Z = strHeader.GetLowerBound(0) To strHeader.GetUpperBound(0)
            If strHeader(Z).StartsWith("Authorization: Basic") Then
                'Sent Auth Info
                AuthEncrypted = Mid(strHeader(Z), 21)
                Dim SB As StringBuilder = New StringBuilder()
                Dim B64() As Byte = Convert.FromBase64String(AuthEncrypted)
                Dim B64Bit As Byte
                For Each B64Bit In B64
                    SB.Append(ChrW(B64Bit))
                Next
                Dim splUserPass() As String = Split(SB.ToString, ":")
                Dim strUser As String = splUserPass(0)
                Dim strPass As String = splUserPass(1)

                If AuthenticateUser(strUser, strPass) Then strOK = True

            End If
        Next

        If strOK = False Then
            'No Authentication Sent
            'Send 401 UNAUTHORIZED message
            Dim strResponse As String = ""
            Console.WriteLine("WebServer: Request Authentication")
            strResponse &= "HTTP/1.0 401 Unauthorized" & vbCrLf
            strResponse &= "WWW-Authenticate: Basic realm = ""BlazeRegulator.NET Web Admin""" & vbCrLf
            strResponse &= vbCrLf
            strResponse &= "401 Unauthorized"
            Me.SendData(strResponse)
            CloseMe()
            Exit Function
        End If

        If strOK = True Then
            'Authentication is OK, Process Request

            Dim ByteLength As Integer
            Dim bIsLocalFile As Boolean
            Dim LocalFileName As String

            Dim strResponse As String = ""
            Dim RequestLine() As String = Split(strHeader(0), " ", -1, CompareMethod.Text)
            Dim RequestMethod As String = RequestLine(0)
            Dim RequestURL As String = RequestLine(1)

            Console.WriteLine("WebServer: " & RequestMethod & " " & RequestURL)

            'Determine Request Type - Get Extension
            Dim RequestType As String = DetermineMIME(RequestURL)
            RequestURL = RequestURL.Replace("%20", " ")

            'Set Status Code to OK
            Me.StatusCode = "HTTP/1.0 200 OK"

            'Get Request Location
            Dim RequestLocation As String = "/"
            If RequestURL.IndexOf("/", 2) > 0 Then
                RequestLocation = Left(RequestURL, RequestURL.IndexOf("/", 2))
            End If

            'Determine Request Type - Get Response
            Select Case RequestLocation
                Case "/"

                Case "/embed"
                    'Embedded File
                    'Dim ClassName As String = RequestURL.Remove(0, RequestURL.LastIndexOf("/") + 1)
                    'ByteLength = System.Text.Encoding.UTF8.GetByteCount(GetEmbeddedFile(ClassName))
                    If Not EmbeddedFileExists(RequestURL.Remove(0, RequestURL.LastIndexOf("/") + 1)) Then
                        StatusCode = "HTTP/1.0 301 Moved Permanently"
                        AppendHeader = "Location: " & vbCrLf
                    End If
                    bIsEmbedded = True
                    'strResponse = GetEmbeddedFile(ClassName)

                Case "/function"
                    'Call Function - Return Response
                    Dim ArrayItems As String
                    Dim ArrayList() As String = {}
                    Dim AItem As Integer
                    Dim FuncName As String = RequestURL

                    If RequestURL.IndexOf("?") > 0 Then
                        ArrayItems = Right(RequestURL, Len(RequestURL) - (RequestURL.IndexOf("?") + 1))
                        ArrayList = Split(ArrayItems, "&")
                        For AItem = ArrayList.GetLowerBound(0) To ArrayList.GetUpperBound(0)
                            ArrayList(AItem) = ArrayList(AItem).Remove(0, ArrayList(AItem).IndexOf("=") + 1)
                        Next
                        FuncName = Left(RequestURL, RequestURL.IndexOf("?"))
                    End If

                    ExposeFunctions = New clsWebExposedProcedures(Me, Nothing)
                    FuncName = FuncName.Remove(0, FuncName.LastIndexOf("/") + 1)
                    strResponse = Me.CallByNameProcedure(FuncName, ArrayList)
                    ExposeFunctions = Nothing

                Case "/hd"
                    'File from Harddrive
                    Dim Folder As String = ""
                    Dim ExposeFolder As String = ""
                    Dim SubFolders As String = ""

                    ExposeFolder = Right(RequestURL, (Len(RequestURL)) - RequestURL.IndexOf("/", 2))
                    If ExposeFolder.EndsWith("/") Then ExposeFolder.Remove(0, 1)

                    If ExposeFolder.IndexOf("/", 2) > 0 Then
                        SubFolders = ExposeFolder.Remove(0, ExposeFolder.IndexOf("/", 2))
                        ExposeFolder = Left(ExposeFolder, ExposeFolder.IndexOf("/", 2))
                    End If

                    Dim Exp As String
                    For Each Exp In Me.HarddrivePaths
                        If Split(Exp, "|")(0) = ExposeFolder Then
                            Folder = Split(Exp, "|")(1)
                        End If
                    Next

                    SubFolders = SubFolders.Replace("/", "\")

                    If Folder <> "" Then
                        Dim F As IO.FileSystemInfo
                        F = New IO.DirectoryInfo(Folder & SubFolders)
                        If F.Exists Then
                            Dim RequestURLFix As String = RequestURL
                            If Not RequestURL.EndsWith("/") Then RequestURLFix &= "/"
                            Try
                                Dim DirInfo As IO.DirectoryInfo = New IO.DirectoryInfo(Folder & SubFolders)
                                DirInfo = Nothing
                                strResponse = HTMLDirList(Folder & SubFolders, RequestURLFix)
                            Catch ua As UnauthorizedAccessException
                                'No Access
                                Me.StatusCode = "HTTP/1.0 403 Forbidden"
                                strResponse = "Error 403 Forbidden"
                            Catch e As Exception
                                'Other Error
                                Me.StatusCode = "HTTP/1.0 404 Not Found"
                                strResponse = "Error 404 Not Found"
                            End Try
                        Else
                            F = New IO.FileInfo(Folder & SubFolders)
                            If F.Exists Then
                                LocalFileName = Folder & SubFolders
                                bIsLocalFile = True
                                'strResponse = HTMLDirList(Folder & SubFolders, "")
                            Else
                                Me.StatusCode = "HTTP/1.0 404 Not Found"
                                strResponse = "Error 404 Not Found"
                            End If
                        End If
                        F = Nothing
                    Else
                        Me.StatusCode = "HTTP/1.0 404 Not Found"
                        strResponse = "Error 404 Not Found"
                    End If


                Case Else
                    'Not Supported
                    Me.StatusCode = "HTTP/1.0 501 Not Implemented"

            End Select

            'Build a Header
            Dim strReturnHeader As String = ""
            strReturnHeader &= Me.StatusCode & vbCrLf
            If bIsEmbedded = False And bIsLocalFile = False Then strReturnHeader &= "Content-Length: " & Len(strResponse) & vbCrLf
            strReturnHeader &= "Content-Type: " & RequestType & vbCrLf
            strReturnHeader &= AppendHeader
            strReturnHeader &= "Server: BRNET/1.2" & vbCrLf
            strReturnHeader &= vbCrLf

            'Attach Header to Response
            'Send Data & Close Connection

            If bIsLocalFile = True Then
                Me.SendData(strReturnHeader)
                SendLocalFile(LocalFileName)
                CloseMe()
            ElseIf bIsEmbedded = False Then
                'strResponse = strReturnHeader & strResponse
                Me.SendData(strReturnHeader)

                nsSocket.Write(Encoding.ASCII.GetBytes(strResponse), 0, strResponse.Length)
                nsSocket.Flush()

                CloseMe()
            Else
                Me.SendData(strReturnHeader)
                SendEmbeddedFile(RequestURL.Remove(0, RequestURL.LastIndexOf("/") + 1))

                'Dim w As New IO.StreamWriter(nsSocket, System.Text.Encoding.UTF8)
                'nsSocket.Write(ByteResponse, 0, ByteResponse.Length)
                'nsSocket.Flush()
                'w.Write(Encoding.UTF8.GetChars(ByteResponse))
                'w.Flush()
                CloseMe()
            End If


        End If

    End Function

    Private Function GetEmbeddedFile(ByVal EmbeddedClass As String) As String
        Dim Asm As System.Reflection.Assembly = System.Reflection.Assembly.GetExecutingAssembly()
        Dim Strm As IO.Stream = Asm.GetManifestResourceStream(Asm.GetName.Name & "." & EmbeddedClass)
        Dim StrmRead As IO.StreamReader

        If Not Strm Is Nothing Then
            StrmRead = New IO.StreamReader(Strm)
            GetEmbeddedFile = StrmRead.ReadToEnd
            StrmRead.Close()
            Strm.Close()
        Else
            Me.StatusCode = "HTTP/1.0 404 Not Found"
            GetEmbeddedFile = "Error 404 Not Found"
        End If


        StrmRead = Nothing
        Strm = Nothing
        Asm = Nothing
    End Function

    Private Function EmbeddedFileExists(ByVal EmbeddedClass As String) As Boolean
        Dim Asm As System.Reflection.Assembly = System.Reflection.Assembly.GetExecutingAssembly()
        Dim Strm As IO.Stream = Asm.GetManifestResourceStream(Asm.GetName.Name & "." & EmbeddedClass)

        If Strm Is Nothing Then
            Return False
        Else
            Return True
        End If

        Strm = Nothing
        Asm = Nothing
    End Function

    Private Function SendEmbeddedFile(ByVal EmbeddedClass As String) As Boolean
        Dim Asm As System.Reflection.Assembly = System.Reflection.Assembly.GetExecutingAssembly()
        Dim Strm As IO.Stream = Asm.GetManifestResourceStream(Asm.GetName.Name & "." & EmbeddedClass)
        Dim StrmRead As IO.StreamReader

        If Not Strm Is Nothing Then
            Dim b(1024) As Byte
            Dim OffsetSize As Integer

            Do
                OffsetSize = Strm.Read(b, 0, 1024)
                nsSocket.Write(b, 0, OffsetSize)
                nsSocket.Flush()
            Loop Until OffsetSize = 0
            Strm.Close()
        Else
            Me.StatusCode = "HTTP/1.0 404 Not Found"
            Me.SendData("Error 404 Not Found")
        End If


        StrmRead = Nothing
        Strm = Nothing
        Asm = Nothing
    End Function

    Private Function SendLocalFile(ByVal FileName As String) As Boolean
        Dim Strm As IO.Stream = New IO.FileStream(FileName, IO.FileMode.Open)
        Dim StrmRead As IO.StreamReader

        If Not Strm Is Nothing Then
            Dim b(1024) As Byte
            Dim OffsetSize As Integer

            Do
                OffsetSize = Strm.Read(b, 0, 1024)
                nsSocket.Write(b, 0, OffsetSize)
                nsSocket.Flush()
            Loop Until OffsetSize = 0
            Strm.Close()
        Else
            Me.StatusCode = "HTTP/1.0 404 Not Found"
            Me.SendData("Error 404 Not Found")
        End If


        StrmRead = Nothing
        Strm = Nothing
    End Function


    Private Function CallByNameProcedure(ByVal ProcName As String, ByVal ParamArray Args() As Object)
        Try
            Return CallByName(ExposeFunctions, ProcName, CallType.Method, Args)

        Catch e As Exception
            Me.StatusCode = "HTTP/1.0 400 Bad Request"
            Return "400 Bad Request" & vbCrLf & vbCrLf & e.Message.ToString
        End Try
    End Function

    Public Function DetermineMIME(ByVal FilenameExtention As String)
        Dim Filename As String = FilenameExtention
        Dim RequestType As String = "application/octet-stream"

        If FilenameExtention.IndexOf("?") > 0 Then
            'Strip Query from Request
            Filename = Left(FilenameExtention, FilenameExtention.IndexOf("?") - 1)
        End If

        'HTML Documents
        If Filename.EndsWith(".html") Then RequestType = "text/html"
        If Filename.EndsWith(".htm") Then RequestType = "text/html"

        'Plain Text
        If Filename.EndsWith(".txt") Then RequestType = "text/plain"
        If Filename.EndsWith(".log") Then RequestType = "text/plain"
        If Filename.EndsWith(".ini") Then RequestType = "text/plain"

        'Image Files
        If Filename.EndsWith(".jpg") Then RequestType = "image/jpeg"
        If Filename.EndsWith(".jpeg") Then RequestType = "image/jpeg"
        If Filename.EndsWith(".jpe") Then RequestType = "image/jpeg"
        If Filename.EndsWith(".gif") Then RequestType = "image/gif"
        If Filename.EndsWith(".png") Then RequestType = "image/x-png"
        If Filename.EndsWith(".bmp") Then RequestType = "image/x-ms-bmp"

        Return RequestType
    End Function

    Private Function HTMLDirList(ByVal Dir As String, ByVal URL As String) As String
        Dim URLCheck As String = URL
        If URL.EndsWith("/") Then URLCheck = Left(URL, Len(URL) - 1)

        Dim strResponse As String = ""
        strResponse &= "<HTML><BODY>"
        strResponse &= "<FONT face=""Courier New"" size=""2"">"
        strResponse &= "<TABLE id=""Table1"" cellSpacing=""0"" cellPadding=""0"" width=""100%"" border=""0"">"
        strResponse &= "<TR>"
        strResponse &= "<TD style=""WIDTH: 50%""><STRONG><FONT size=""2"">Filename</FONT></STRONG></TD>"
        strResponse &= "<TD style=""WIDTH: 25%""><STRONG><FONT size=""2"">Size (Bytes)</FONT></STRONG></TD>"
        strResponse &= "<TD><STRONG><FONT size=""2"">Date</FONT></STRONG></TD>"
        strResponse &= "</TR>"

        Dim DirInfo As IO.DirectoryInfo = New IO.DirectoryInfo(Dir)
        Dim Directories() As IO.DirectoryInfo = DirInfo.GetDirectories()
        Dim Files() As IO.FileInfo = DirInfo.GetFiles()

        Dim Directory As IO.DirectoryInfo
        Dim File As IO.FileInfo

        For Each Directory In Directories
            strResponse &= "<TR>"
            strResponse &= "<TD style=""WIDTH: 50%""><FONT size=""2""><a href=""" & URL & Directory.Name & "/"">" & Directory.Name & "</a></FONT></TD>"
            strResponse &= "<TD style=""WIDTH: 25%""><FONT size=""2"">Directory</FONT></TD>"
            strResponse &= "<TD><FONT size=""2""></FONT></TD>"
            strResponse &= "</TR>"
        Next

        For Each File In Files
            strResponse &= "<TR>"
            strResponse &= "<TD style=""WIDTH: 50%""><FONT size=""2""><a href=""" & URL & File.Name & """>" & File.Name & "</a></FONT></TD>"
            strResponse &= "<TD style=""WIDTH: 25%""><FONT size=""2"">" & File.Length & "</FONT></TD>"
            strResponse &= "<TD><FONT size=""2"">" & File.LastWriteTime & "</FONT></TD>"
            strResponse &= "</TR>"
        Next

        strResponse &= "</TABLE>"
        strResponse &= "</FONT>"
        strResponse &= "</BODY></HTML>"

        Return strResponse
    End Function

#End Region

End Class

Public Class clsWebExposedProcedures

    Private WCLink As clsWebClient
    Private AppLink As Object

    Public Sub New(ByVal WebClientLink As clsWebClient, ByVal ApplicationLink As Object)
        WCLink = WebClientLink
        AppLink = ApplicationLink
    End Sub

    'Public Function RestartBR() As String
    '    Shell(Application.ExecutablePath & " /autoconnect")
    '    Application.Exit()
    '    End
    'End Function

End Class



